package com.sun.imageio.plugins.gif;

import javax.imageio.metadata.IIOInvalidTreeException;
import java.util.Iterator;
import javax.imageio.metadata.IIOMetadataNode;
import java.io.UnsupportedEncodingException;
import org.w3c.dom.Node;
import java.util.List;

public class GIFImageMetadata extends GIFMetadata
{
    static final String nativeMetadataFormatName = "javax_imageio_gif_image_1.0";
    static final String[] disposalMethodNames;
    public int imageLeftPosition;
    public int imageTopPosition;
    public int imageWidth;
    public int imageHeight;
    public boolean interlaceFlag;
    public boolean sortFlag;
    public byte[] localColorTable;
    public int disposalMethod;
    public boolean userInputFlag;
    public boolean transparentColorFlag;
    public int delayTime;
    public int transparentColorIndex;
    public boolean hasPlainTextExtension;
    public int textGridLeft;
    public int textGridTop;
    public int textGridWidth;
    public int textGridHeight;
    public int characterCellWidth;
    public int characterCellHeight;
    public int textForegroundColor;
    public int textBackgroundColor;
    public byte[] text;
    public List applicationIDs;
    public List authenticationCodes;
    public List applicationData;
    public List comments;
    
    protected GIFImageMetadata(boolean b, String s, String s2, String[] array, String[] array2) {
        super(b, s, s2, array, array2);
        this.interlaceFlag = false;
        this.sortFlag = false;
        this.localColorTable = null;
        this.disposalMethod = 0;
        this.userInputFlag = false;
        this.transparentColorFlag = false;
        this.delayTime = 0;
        this.transparentColorIndex = 0;
        this.hasPlainTextExtension = false;
        this.applicationIDs = null;
        this.authenticationCodes = null;
        this.applicationData = null;
        this.comments = null;
    }
    
    public GIFImageMetadata() {
        this(true, "javax_imageio_gif_image_1.0", "com.sun.imageio.plugins.gif.GIFImageMetadataFormat", null, (String[])null);
    }
    
    @Override
    public boolean isReadOnly() {
        return true;
    }
    
    @Override
    public Node getAsTree(String s) {
        if (s.equals("javax_imageio_gif_image_1.0")) {
            return this.getNativeTree();
        }
        if (s.equals("javax_imageio_1.0")) {
            return this.getStandardTree();
        }
        throw new IllegalArgumentException("Not a recognized format!");
    }
    
    private String toISO8859(byte[] bytes) {
        try {
            return new String(bytes, "ISO-8859-1");
        }
        catch (UnsupportedEncodingException ex) {
            return "";
        }
    }
    
    private Node getNativeTree() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("javax_imageio_gif_image_1.0");
        IIOMetadataNode newChild = new IIOMetadataNode("ImageDescriptor");
        newChild.setAttribute("imageLeftPosition", Integer.toString(this.imageLeftPosition));
        newChild.setAttribute("imageTopPosition", Integer.toString(this.imageTopPosition));
        newChild.setAttribute("imageWidth", Integer.toString(this.imageWidth));
        newChild.setAttribute("imageHeight", Integer.toString(this.imageHeight));
        newChild.setAttribute("interlaceFlag", this.interlaceFlag ? "TRUE" : "FALSE");
        iioMetadataNode.appendChild(newChild);
        if (this.localColorTable != null) {
            IIOMetadataNode newChild2 = new IIOMetadataNode("LocalColorTable");
            int i = this.localColorTable.length / 3;
            newChild2.setAttribute("sizeOfLocalColorTable", Integer.toString(i));
            newChild2.setAttribute("sortFlag", this.sortFlag ? "TRUE" : "FALSE");
            for (int j = 0; j < i; ++j) {
                IIOMetadataNode newChild3 = new IIOMetadataNode("ColorTableEntry");
                newChild3.setAttribute("index", Integer.toString(j));
                int k = this.localColorTable[3 * j] & 0xFF;
                int l = this.localColorTable[3 * j + 1] & 0xFF;
                int m = this.localColorTable[3 * j + 2] & 0xFF;
                newChild3.setAttribute("red", Integer.toString(k));
                newChild3.setAttribute("green", Integer.toString(l));
                newChild3.setAttribute("blue", Integer.toString(m));
                newChild2.appendChild(newChild3);
            }
            iioMetadataNode.appendChild(newChild2);
        }
        IIOMetadataNode newChild4 = new IIOMetadataNode("GraphicControlExtension");
        newChild4.setAttribute("disposalMethod", GIFImageMetadata.disposalMethodNames[this.disposalMethod]);
        newChild4.setAttribute("userInputFlag", this.userInputFlag ? "TRUE" : "FALSE");
        newChild4.setAttribute("transparentColorFlag", this.transparentColorFlag ? "TRUE" : "FALSE");
        newChild4.setAttribute("delayTime", Integer.toString(this.delayTime));
        newChild4.setAttribute("transparentColorIndex", Integer.toString(this.transparentColorIndex));
        iioMetadataNode.appendChild(newChild4);
        if (this.hasPlainTextExtension) {
            IIOMetadataNode newChild5 = new IIOMetadataNode("PlainTextExtension");
            newChild5.setAttribute("textGridLeft", Integer.toString(this.textGridLeft));
            newChild5.setAttribute("textGridTop", Integer.toString(this.textGridTop));
            newChild5.setAttribute("textGridWidth", Integer.toString(this.textGridWidth));
            newChild5.setAttribute("textGridHeight", Integer.toString(this.textGridHeight));
            newChild5.setAttribute("characterCellWidth", Integer.toString(this.characterCellWidth));
            newChild5.setAttribute("characterCellHeight", Integer.toString(this.characterCellHeight));
            newChild5.setAttribute("textForegroundColor", Integer.toString(this.textForegroundColor));
            newChild5.setAttribute("textBackgroundColor", Integer.toString(this.textBackgroundColor));
            newChild5.setAttribute("text", this.toISO8859(this.text));
            iioMetadataNode.appendChild(newChild5);
        }
        int n = (this.applicationIDs == null) ? 0 : this.applicationIDs.size();
        if (n > 0) {
            IIOMetadataNode newChild6 = new IIOMetadataNode("ApplicationExtensions");
            for (int n2 = 0; n2 < n; ++n2) {
                IIOMetadataNode newChild7 = new IIOMetadataNode("ApplicationExtension");
                newChild7.setAttribute("applicationID", this.toISO8859((byte[])(byte[])this.applicationIDs.get(n2)));
                newChild7.setAttribute("authenticationCode", this.toISO8859((byte[])(byte[])this.authenticationCodes.get(n2)));
                newChild7.setUserObject((byte[])(byte[])((byte[])(byte[])this.applicationData.get(n2)).clone());
                newChild6.appendChild(newChild7);
            }
            iioMetadataNode.appendChild(newChild6);
        }
        int n3 = (this.comments == null) ? 0 : this.comments.size();
        if (n3 > 0) {
            IIOMetadataNode newChild8 = new IIOMetadataNode("CommentExtensions");
            for (int n4 = 0; n4 < n3; ++n4) {
                IIOMetadataNode newChild9 = new IIOMetadataNode("CommentExtension");
                newChild9.setAttribute("value", this.toISO8859((byte[])(byte[])this.comments.get(n4)));
                newChild8.appendChild(newChild9);
            }
            iioMetadataNode.appendChild(newChild8);
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardChromaNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Chroma");
        IIOMetadataNode newChild = new IIOMetadataNode("ColorSpaceType");
        newChild.setAttribute("name", "RGB");
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("NumChannels");
        newChild2.setAttribute("value", this.transparentColorFlag ? "4" : "3");
        iioMetadataNode.appendChild(newChild2);
        IIOMetadataNode newChild3 = new IIOMetadataNode("BlackIsZero");
        newChild3.setAttribute("value", "TRUE");
        iioMetadataNode.appendChild(newChild3);
        if (this.localColorTable != null) {
            IIOMetadataNode newChild4 = new IIOMetadataNode("Palette");
            for (int n = this.localColorTable.length / 3, i = 0; i < n; ++i) {
                IIOMetadataNode newChild5 = new IIOMetadataNode("PaletteEntry");
                newChild5.setAttribute("index", Integer.toString(i));
                newChild5.setAttribute("red", Integer.toString(this.localColorTable[3 * i] & 0xFF));
                newChild5.setAttribute("green", Integer.toString(this.localColorTable[3 * i + 1] & 0xFF));
                newChild5.setAttribute("blue", Integer.toString(this.localColorTable[3 * i + 2] & 0xFF));
                newChild4.appendChild(newChild5);
            }
            iioMetadataNode.appendChild(newChild4);
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardCompressionNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Compression");
        IIOMetadataNode newChild = new IIOMetadataNode("CompressionTypeName");
        newChild.setAttribute("value", "lzw");
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("Lossless");
        newChild2.setAttribute("value", "TRUE");
        iioMetadataNode.appendChild(newChild2);
        IIOMetadataNode newChild3 = new IIOMetadataNode("NumProgressiveScans");
        newChild3.setAttribute("value", this.interlaceFlag ? "4" : "1");
        iioMetadataNode.appendChild(newChild3);
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardDataNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Data");
        IIOMetadataNode newChild = new IIOMetadataNode("SampleFormat");
        newChild.setAttribute("value", "Index");
        iioMetadataNode.appendChild(newChild);
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardDimensionNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Dimension");
        IIOMetadataNode newChild = new IIOMetadataNode("ImageOrientation");
        newChild.setAttribute("value", "Normal");
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("HorizontalPixelOffset");
        newChild2.setAttribute("value", Integer.toString(this.imageLeftPosition));
        iioMetadataNode.appendChild(newChild2);
        IIOMetadataNode newChild3 = new IIOMetadataNode("VerticalPixelOffset");
        newChild3.setAttribute("value", Integer.toString(this.imageTopPosition));
        iioMetadataNode.appendChild(newChild3);
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardTextNode() {
        if (this.comments == null) {
            return null;
        }
        Iterator<Object> iterator = this.comments.iterator();
        if (!iterator.hasNext()) {
            return null;
        }
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Text");
        while (iterator.hasNext()) {
            byte[] bytes = (byte[])(byte[])iterator.next();
            Object value;
            try {
                value = new String(bytes, "ISO-8859-1");
            }
            catch (UnsupportedEncodingException ex) {
                throw new RuntimeException("Encoding ISO-8859-1 unknown!");
            }
            IIOMetadataNode newChild = new IIOMetadataNode("TextEntry");
            newChild.setAttribute("value", value);
            newChild.setAttribute("encoding", "ISO-8859-1");
            newChild.setAttribute("compression", "none");
            iioMetadataNode.appendChild(newChild);
        }
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardTransparencyNode() {
        if (!this.transparentColorFlag) {
            return null;
        }
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Transparency");
        IIOMetadataNode newChild = new IIOMetadataNode("TransparentIndex");
        newChild.setAttribute("value", Integer.toString(this.transparentColorIndex));
        iioMetadataNode.appendChild(newChild);
        return iioMetadataNode;
    }
    
    @Override
    public void setFromTree(String s, Node node) throws IIOInvalidTreeException {
        throw new IllegalStateException("Metadata is read-only!");
    }
    
    @Override
    protected void mergeNativeTree(Node node) throws IIOInvalidTreeException {
        throw new IllegalStateException("Metadata is read-only!");
    }
    
    @Override
    protected void mergeStandardTree(Node node) throws IIOInvalidTreeException {
        throw new IllegalStateException("Metadata is read-only!");
    }
    
    @Override
    public void reset() {
        throw new IllegalStateException("Metadata is read-only!");
    }
    
    static {
        disposalMethodNames = new String[] { "none", "doNotDispose", "restoreToBackgroundColor", "restoreToPrevious", "undefinedDisposalMethod4", "undefinedDisposalMethod5", "undefinedDisposalMethod6", "undefinedDisposalMethod7" };
    }
}
